@@ -349,6 +349,14 @@ tp_dbus_check_valid_interface_name (const gchar *name,
                   name);
               return FALSE;
             }
+          else if (last == '.')
+            {
+              g_set_error (error, TP_DBUS_ERRORS,
+                  TP_DBUS_ERROR_INVALID_INTERFACE_NAME,
+                  "Invalid interface name '%s': a digit must not follow '.'",
+                  name);
+              return FALSE;
+            }
         }
       else if (!g_ascii_isalpha (*ptr) && *ptr != '_')
         {
@@ -926,3 +934,608 @@ tp_dbus_daemon_class_init (TpDBusDaemonClass *klass)
 
 /* Auto-generated implementation of _tp_register_dbus_glib_marshallers */
 #include "_gen/register-dbus-glib-marshallers-body.h"
+
+
+/**
+ * tp_asv_get_boolean:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location to store %TRUE if the key actually
+ *  exists and has a boolean value
+ *
+ * If a value for @key in @asv is present and boolean, return it,
+ * and set *@valid to %TRUE if @valid is not %NULL.
+ *
+ * Otherwise return %FALSE, and set *@valid to %FALSE if @valid is not %NULL.
+ *
+ * (FIXME: should we also allow 'i' and 'u' with nonzero <=> True?)
+ *
+ * Returns: a boolean value for @key
+ * @since 0.7.9
+ */
+gboolean
+tp_asv_get_boolean (const GHashTable *asv,
+                    const gchar *key,
+                    gboolean *valid)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL || !G_VALUE_HOLDS_BOOLEAN (value))
+    {
+      if (valid != NULL)
+        *valid = FALSE;
+
+      return FALSE;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return g_value_get_boolean (value);
+}
+
+
+/**
+ * tp_asv_get_bytes:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ *
+ * If a value for @key in @asv is present and is an array of bytes
+ * (its GType is %DBUS_TYPE_G_UCHAR_ARRAY), return it.
+ *
+ * Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it with
+ * g_boxed_copy (DBUS_TYPE_G_UCHAR_ARRAY, ...) if you need to keep
+ * it for longer.
+ *
+ * Returns: the string value of @key, or %NULL
+ * @since 0.7.9
+ */
+const GArray *
+tp_asv_get_bytes (const GHashTable *asv,
+                   const gchar *key)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL || !G_VALUE_HOLDS (value, DBUS_TYPE_G_UCHAR_ARRAY))
+    return NULL;
+
+  return g_value_get_boxed (value);
+}
+
+
+/**
+ * tp_asv_get_string:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ *
+ * If a value for @key in @asv is present and is a string, return it.
+ *
+ * Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it with g_strdup() if you
+ * need to keep it for longer.
+ *
+ * Returns: the string value of @key, or %NULL
+ * @since 0.7.9
+ */
+const gchar *
+tp_asv_get_string (const GHashTable *asv,
+                   const gchar *key)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL || !G_VALUE_HOLDS_STRING (value))
+    return NULL;
+
+  return g_value_get_string (value);
+}
+
+
+/**
+ * tp_asv_get_int32:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location in which to store %TRUE on success or
+ *    %FALSE on failure
+ *
+ * If a value for @key in @asv is present, has an integer type used by
+ * dbus-glib (guchar, gint, guint, gint64 or guint64) and fits in the
+ * range of a gint32, return it, and if @valid is not %NULL, set *@valid to
+ * %TRUE.
+ *
+ * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
+ *
+ * Returns: the 32-bit signed integer value of @key, or 0
+ * @since 0.7.9
+ */
+gint32
+tp_asv_get_int32 (const GHashTable *asv,
+                  const gchar *key,
+                  gboolean *valid)
+{
+  gint64 i;
+  guint64 u;
+  gint32 ret;
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL)
+    goto return_invalid;
+
+  switch (G_VALUE_TYPE (value))
+    {
+    case G_TYPE_UCHAR:
+      ret = g_value_get_uchar (value);
+      break;
+
+    case G_TYPE_UINT:
+      u = g_value_get_uint (value);
+
+      if (G_UNLIKELY (u > G_MAXINT32))
+        goto return_invalid;
+
+      ret = u;
+      break;
+
+    case G_TYPE_INT:
+      ret = g_value_get_int (value);
+      break;
+
+    case G_TYPE_INT64:
+      i = g_value_get_int64 (value);
+
+      if (G_UNLIKELY (i < G_MININT32 || i > G_MAXINT32))
+        goto return_invalid;
+
+      ret = i;
+      break;
+
+    case G_TYPE_UINT64:
+      u = g_value_get_uint64 (value);
+
+      if (G_UNLIKELY (u > G_MAXINT32))
+        goto return_invalid;
+
+      ret = u;
+      break;
+
+    default:
+      goto return_invalid;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return ret;
+
+return_invalid:
+  if (valid != NULL)
+    *valid = FALSE;
+
+  return 0;
+}
+
+
+/**
+ * tp_asv_get_uint32:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location in which to store %TRUE on success or
+ *    %FALSE on failure
+ *
+ * If a value for @key in @asv is present, has an integer type used by
+ * dbus-glib (guchar, gint, guint, gint64 or guint64) and fits in the
+ * range of a guint32, return it, and if @valid is not %NULL, set *@valid to
+ * %TRUE.
+ *
+ * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
+ *
+ * Returns: the 32-bit unsigned integer value of @key, or 0
+ * @since 0.7.9
+ */
+guint32
+tp_asv_get_uint32 (const GHashTable *asv,
+                   const gchar *key,
+                   gboolean *valid)
+{
+  gint64 i;
+  guint64 u;
+  guint32 ret;
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL)
+    goto return_invalid;
+
+  switch (G_VALUE_TYPE (value))
+    {
+    case G_TYPE_UCHAR:
+      ret = g_value_get_uchar (value);
+      break;
+
+    case G_TYPE_UINT:
+      ret = g_value_get_uint (value);
+      break;
+
+    case G_TYPE_INT:
+      i = g_value_get_int (value);
+
+      if (G_UNLIKELY (i < 0))
+        goto return_invalid;
+
+      ret = i;
+      break;
+
+    case G_TYPE_INT64:
+      i = g_value_get_int64 (value);
+
+      if (G_UNLIKELY (i < 0 || i > G_MAXUINT32))
+        goto return_invalid;
+
+      ret = i;
+      break;
+
+    case G_TYPE_UINT64:
+      u = g_value_get_uint64 (value);
+
+      if (G_UNLIKELY (u > G_MAXUINT32))
+        goto return_invalid;
+
+      ret = u;
+      break;
+
+    default:
+      goto return_invalid;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return ret;
+
+return_invalid:
+  if (valid != NULL)
+    *valid = FALSE;
+
+  return 0;
+}
+
+
+/**
+ * tp_asv_get_int64:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location in which to store %TRUE on success or
+ *    %FALSE on failure
+ *
+ * If a value for @key in @asv is present, has an integer type used by
+ * dbus-glib (guchar, gint, guint, gint64 or guint64) and fits in the
+ * range of a gint64, return it, and if @valid is not %NULL, set *@valid to
+ * %TRUE.
+ *
+ * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
+ *
+ * Returns: the 64-bit signed integer value of @key, or 0
+ * @since 0.7.9
+ */
+gint64
+tp_asv_get_int64 (const GHashTable *asv,
+                  const gchar *key,
+                  gboolean *valid)
+{
+  gint64 ret;
+  guint64 u;
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL)
+    goto return_invalid;
+
+  switch (G_VALUE_TYPE (value))
+    {
+    case G_TYPE_UCHAR:
+      ret = g_value_get_uchar (value);
+      break;
+
+    case G_TYPE_UINT:
+      ret = g_value_get_uint (value);
+      break;
+
+    case G_TYPE_INT:
+      ret = g_value_get_int (value);
+      break;
+
+    case G_TYPE_INT64:
+      ret = g_value_get_int64 (value);
+      break;
+
+    case G_TYPE_UINT64:
+      u = g_value_get_uint64 (value);
+
+      if (G_UNLIKELY (u > G_MAXINT64))
+        goto return_invalid;
+
+      ret = u;
+      break;
+
+    default:
+      goto return_invalid;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return ret;
+
+return_invalid:
+  if (valid != NULL)
+    *valid = FALSE;
+
+  return 0;
+}
+
+
+/**
+ * tp_asv_get_uint64:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location in which to store %TRUE on success or
+ *    %FALSE on failure
+ *
+ * If a value for @key in @asv is present, has an integer type used by
+ * dbus-glib (guchar, gint, guint, gint64 or guint64) and is non-negative,
+ * return it, and if @valid is not %NULL, set *@valid to %TRUE.
+ *
+ * Otherwise, return 0, and if @valid is not %NULL, set *@valid to %FALSE.
+ *
+ * Returns: the 64-bit unsigned integer value of @key, or 0
+ * @since 0.7.9
+ */
+guint64
+tp_asv_get_uint64 (const GHashTable *asv,
+                   const gchar *key,
+                   gboolean *valid)
+{
+  gint64 tmp;
+  guint64 ret;
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL)
+    goto return_invalid;
+
+  switch (G_VALUE_TYPE (value))
+    {
+    case G_TYPE_UCHAR:
+      ret = g_value_get_uchar (value);
+      break;
+
+    case G_TYPE_UINT:
+      ret = g_value_get_uint (value);
+      break;
+
+    case G_TYPE_INT:
+      tmp = g_value_get_int (value);
+
+      if (G_UNLIKELY (tmp < 0))
+        goto return_invalid;
+
+      ret = tmp;
+      break;
+
+    case G_TYPE_INT64:
+      tmp = g_value_get_int64 (value);
+
+      if (G_UNLIKELY (tmp < 0))
+        goto return_invalid;
+
+      ret = tmp;
+      break;
+
+    case G_TYPE_UINT64:
+      ret = g_value_get_uint64 (value);
+      break;
+
+    default:
+      goto return_invalid;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return ret;
+
+return_invalid:
+  if (valid != NULL)
+    *valid = FALSE;
+
+  return 0;
+}
+
+
+/* FIXME: reviewers: should this succeed on all numeric types, or just on
+ * doubles? */
+/**
+ * tp_asv_get_double:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @valid: Either %NULL, or a location in which to store %TRUE on success or
+ *    %FALSE on failure
+ *
+ * If a value for @key in @asv is present and has any numeric type used by
+ * dbus-glib (guchar, gint, guint, gint64, guint64 or gdouble),
+ * return it as a double, and if @valid is not %NULL, set *@valid to %TRUE.
+ *
+ * Otherwise, return 0.0, and if @valid is not %NULL, set *@valid to %FALSE.
+ *
+ * Returns: the double precision floating-point value of @key, or 0.0
+ * @since 0.7.9
+ */
+gdouble
+tp_asv_get_double (const GHashTable *asv,
+                   const gchar *key,
+                   gboolean *valid)
+{
+  gdouble ret;
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL)
+    goto return_invalid;
+
+  switch (G_VALUE_TYPE (value))
+    {
+    case G_TYPE_DOUBLE:
+      ret = g_value_get_double (value);
+      break;
+
+    case G_TYPE_UCHAR:
+      ret = g_value_get_uchar (value);
+      break;
+
+    case G_TYPE_UINT:
+      ret = g_value_get_uint (value);
+      break;
+
+    case G_TYPE_INT:
+      ret = g_value_get_int (value);
+      break;
+
+    case G_TYPE_INT64:
+      ret = g_value_get_int64 (value);
+      break;
+
+    case G_TYPE_UINT64:
+      ret = g_value_get_uint64 (value);
+      break;
+
+    default:
+      goto return_invalid;
+    }
+
+  if (valid != NULL)
+    *valid = TRUE;
+
+  return ret;
+
+return_invalid:
+  if (valid != NULL)
+    *valid = FALSE;
+
+  return 0;
+}
+
+
+/**
+ * tp_asv_get_object_path:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ *
+ * If a value for @key in @asv is present and is an object path, return it.
+ *
+ * Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it with g_strdup() if you
+ * need to keep it for longer.
+ *
+ * Returns: the object-path value of @key, or %NULL
+ * @since 0.7.9
+ */
+const gchar *
+tp_asv_get_object_path (const GHashTable *asv,
+                        const gchar *key)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL || !G_VALUE_HOLDS (value, DBUS_TYPE_G_OBJECT_PATH))
+    return NULL;
+
+  return g_value_get_boxed (value);
+}
+
+
+/**
+ * tp_asv_get_boxed:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ * @type: The type that the key's value should have, which must be derived
+ *  from %G_TYPE_BOXED
+ *
+ * If a value for @key in @asv is present and is of the desired type,
+ * return it.
+ *
+ * Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it, for instance with
+ * g_boxed_copy(), if you need to keep it for longer.
+ *
+ * Returns: the value of @key, or %NULL
+ * @since 0.7.9
+ */
+gpointer
+tp_asv_get_boxed (const GHashTable *asv,
+                  const gchar *key,
+                  GType type)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  g_return_val_if_fail (G_TYPE_FUNDAMENTAL (type) == G_TYPE_BOXED, NULL);
+
+  if (value == NULL || !G_VALUE_HOLDS (value, type))
+    return NULL;
+
+  return g_value_get_boxed (value);
+}
+
+
+/**
+ * tp_asv_get_strv:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ *
+ * If a value for @key in @asv is present and is an array of strings (strv),
+ * return it.
+ *
+ * Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it with g_strdupv() if you
+ * need to keep it for longer.
+ *
+ * Returns: the %NULL-terminated string-array value of @key, or %NULL
+ * @since 0.7.9
+ */
+const gchar * const *
+tp_asv_get_strv (const GHashTable *asv,
+                 const gchar *key)
+{
+  GValue *value = g_hash_table_lookup ((GHashTable *) asv, key);
+
+  if (value == NULL || !G_VALUE_HOLDS (value, G_TYPE_STRV))
+    return NULL;
+
+  return g_value_get_boxed (value);
+}
+
+
+/**
+ * tp_asv_lookup:
+ * @asv: A GHashTable where the keys are strings and the values are GValues
+ * @key: The key to look up
+ *
+ * If a value for @key in @asv is present, return it. Otherwise return %NULL.
+ *
+ * The returned value is not copied, and is only valid as long as the value
+ * for @key in @asv is not removed or altered. Copy it with (for instance)
+ * g_value_copy() if you need to keep it for longer.
+ *
+ * Returns: the value of @key, or %NULL
+ * @since 0.7.9
+ */
+const GValue *
+tp_asv_lookup (const GHashTable *asv,
+               const gchar *key)
+{
+  return g_hash_table_lookup ((GHashTable *) asv, key);
+}
